classdef Knapsack
    % KNAPSACK Stores problem data and calculates features of 0-1KP instance
    % Authors: Jeffrey Christiansen, Samuel Fairchild
    
    properties
        Name
        Class
    end
    
    properties (SetAccess = private)
        Weights
        Values
        Capacity
        LinearBound
        GreedyBound
        SplitRatio
        GreedyWaste
        WSortedEffic
        VSortedEffic
        SortedEfficInd
        ESortedEffic
        WSortedWeight
        VSortedWeight
        SortedWeightInd
        WSortedValue
        VSortedValue
        SortedValueInd
        Fixes
    end
    
     properties (Dependent = true)
         Size
     end
    
    methods
        
        %------------------------------------------------------------------
        % Constructor and file I/O
        %------------------------------------------------------------------
        
        function KP = Knapsack(Name, Class, Weights, Values, Capacity)
            % Constructor
            KP.Name = Name;
            KP.Class = Class;
            if size(Weights,2) > 1
                Weights = Weights';
            end
            if size(Values,2) > 1
                Values = Values';
            end
            KP.Weights = Weights;
            KP.Values = Values;
            KP.Capacity = Capacity;
            E = KP.Values./KP.Weights;
            KP.ESortedEffic = sort(E);
            [KP.WSortedEffic, KP.VSortedEffic, KP.SortedEfficInd] = KP.sortedByRatio;
            [KP.WSortedWeight, KP.VSortedWeight, KP.SortedWeightInd] = KP.sortedByWeight;
            [KP.WSortedValue, KP.VSortedValue, KP.SortedValueInd] = KP.sortedByValue;
            [KP.LinearBound, KP.GreedyBound, KP.SplitRatio, KP.GreedyWaste] = KP.simpleBounds;
            KP.Fixes = KP.findFixes;
        end
        
        function writeToFile(KP, path)
            %WRITETOFILE write knapsack instance to file
            % Save knapsack to file, used by instancegen/createInstances.m
            % The corresponding readFromFile is a static method, see the
            % end of this file.
            f = fopen(strcat(path, KP.Name), 'w');
            fprintf(f, '%s\n', KP.Name);
            fprintf(f, '%s\n', KP.Class);
            fprintf(f,'%d\n',KP.Capacity);
            fprintf(f,'%d\t%d\n',[KP.Weights,KP.Values]');
            fclose(f);
        end
        
        function [expTime, minTime, comTime, solution] = readSolutions(KP, exp_file, min_file, com_file)
            %READSOLUTIONS Read algorithms' performance from file
            % Read from file the time required for algorithms to solve this
            % knapsack instance, and the solution found by Combo.
            
            expTime = 1000000000;
            minTime = 1000000000;
            comTime = 1000000000;
            solution = -1; % Default value
			expsolution = -1;
			minsolution = -1;
            
            % get runtimes from expknap
            f = fopen(exp_file); %file openned
            tline = fgetl(f); %check first line in file
            found = false; %initialises the 'found' variable
            
            while ischar(tline) && ~found
                % Search through the file until this instance is found.
                % This is rather inefficient if we are dealing with more
                % than one instance but the job gets done.
                fline = strread(tline, '%s', -1, 'delimiter', ',');
                if strcmp(fline{1},KP.Name)
                    expTime = str2double(fline{2});
					expsolution = str2double(fline{3}); 
                    found = true;
                end
                tline = fgetl(f);
            end
            fclose(f);
            
            f = fopen(min_file);
            tline = fgetl(f);
            found = false;
            while ischar(tline) && ~found
                fline = strread(tline, '%s', -1, 'delimiter', ',');
                if strcmp(fline{1},KP.Name)
                    minTime = str2double(fline{2});
					minsolution = str2double(fline{3});
                    found = true;
                end
                tline = fgetl(f);
            end
            fclose(f);
            
            f = fopen(com_file);
            tline = fgetl(f);
            found = false;
            while ischar(tline) && ~found
                fline = strread(tline, '%s', -1, 'delimiter', ',');
                if strcmp(fline{1},KP.Name)
                    comTime = str2double(fline{2});
                    solution = str2double(fline{3}); %grabs solution as well
                    found = true;
                end
                tline = fgetl(f);
            end
            fclose(f);
        end
        
        function s = get.Size(KP)
            % Returns the number of items in the knapsack.
            s = length(KP.Weights);
        end
        
        %------------------------------------------------------------------
        % Toolbox functions contributing to feature extratction
        %------------------------------------------------------------------
        
        function [LinearRelaxation, GreedySolution, SplitRatio, GreedyWaste] = simpleBounds(KP)
            %SIMPLEBOUNDS Calculates basic upper/lower bounds
            % Returns the objective value of the greedy solution, the
            % linear relaxation of the KP instance, and the efficiency of
            % the final item which is (potentially partially) included in
            % the LR solution.
            % This function is called several times in the course of
            % generating all features.
            SortedW = KP.WSortedEffic;
            SortedV = KP.VSortedEffic;
            tmpweight = 0;
            tmpvalue = 0;
            i = 1;
            
            if KP.Size > 0
                while (i <= KP.Size) && (tmpweight + SortedW(i) <= KP.Capacity)
                    tmpweight = tmpweight + SortedW(i);
                    tmpvalue = tmpvalue + SortedV(i);
                    i = i + 1;
                end
                
                if i <= KP.Size
                    LinearRelaxation = tmpvalue + SortedV(i) * (KP.Capacity - tmpweight) / SortedW(i);
                    if (KP.Capacity - tmpweight == 0) && (i > 1)
                        % If the linear relaxation exactly fills the
                        % knapsack, set the split ratio in between the
                        % efficiencies of the borderline items. This allows
                        % us to fix them in or out immediately.
                        SplitRatio = 0.5*(SortedV(i)/SortedW(i) + SortedV(i-1)/SortedW(i-1));
                    else
                        SplitRatio = SortedV(i)/SortedW(i);
                    end
                else
                    LinearRelaxation = tmpvalue;
                    SplitRatio = SortedV(KP.Size)/SortedW(KP.Size);
                end
                
                while (i <= KP.Size)
                    if (tmpweight + SortedW(i)) <= KP.Capacity
                        tmpweight = tmpweight + SortedW(i);
                        tmpvalue = tmpvalue + SortedV(i);
                    end
                    i = i + 1;
                end
                GreedySolution = tmpvalue;
                GreedyWaste = KP.Capacity - tmpweight;
            else
                LinearRelaxation = 0;
                GreedySolution = 0;
                SplitRatio = 0;
                GreedyWaste = 0;
            end
        end
        
        function [SortedWeights, SortedValues, IX] = sortedByWeight(KP)
            %SORTEDBYWEIGHT Returns item list sorted by weight
            % Returns a list of weights and values corresponding to the
            % knapsack items sorted by weight.
            [~, IX] = sort(KP.Weights);
            SortedWeights = KP.Weights(IX);
            SortedValues = KP.Values(IX);
        end
        
        function [SortedWeights, SortedValues, IX] = sortedByValue(KP)
            %SORTEDBYVALUE Returns item list sorted by profit
            % Returns a list of weights and values corresponding to the
            % knapsack items sorted by weight.
            [~, IX] = sort(KP.Values);
            SortedWeights = KP.Weights(IX);
            SortedValues = KP.Values(IX);
        end
        
        function [SortedWeights, SortedValues, IX] = sortedByRatio(KP)
            %SORTEDBYRATIO Returns item list sorted by efficiency
            % Returns a list of weights and values corresponding to the
            % knapsack items sorted by efficiency (weight/value).
            [~, IX] = sort(KP.Weights ./ (KP.Values + 1e-10)) ; % gives order from lowest weight/value ('best' item) to highest weight/value ('worst' item)
            [SortedWeights] = KP.Weights(IX);
            [SortedValues] = KP.Values(IX);
        end
        
        function fixes = findFixes(KP)
            %FINDFIXES Determines which items can be "trivially fixed"
            % Determines which knapsack items can be "trivially" assigned
            % as in or out of the optimal packing based on the greedy
            % solution and linear relaxation bound.
            % Returns a vector of values corresponding to the KP items,
            % with each entry equal to:
            % 1 if the item can be trivially assigned as being IN the
            % optimal packing
            % -1 if the item can be trivially assigned as being OUT of the
            % optimal packing
            % 0 if the item cannot be trivially assigned
            
            LR = KP.LinearBound;
            GS = KP.GreedyBound;
            SR = KP.SplitRatio;
            
            fixes = zeros(KP.Size, 1);
            
            for i = 1:KP.Size
                if KP.Values(i)/KP.Weights(i) > SR
                    % This item is currently in our linear relaxation
                    % solution. What upper bound do we have if we take it
                    % out? Assume we don't have to put anything less efficient
                    % than the split item in (this assumption weakens the bound).
                    fixOutUB = LR - KP.Values(i) + KP.Weights(i)*SR;
                    if fixOutUB <= GS
                        % Fixing the item to be out cannot result in a
                        % better solution than the greedy solution, so fix
                        % it in.
                        fixes(i) = 1;
                    end
                end
                if KP.Values(i)/KP.Weights(i) < SR
                    % This item is currently not in our linear relaxation
                    % solution. What upper bound do we have if we force this item to
                    % be in? Assume we don't have to take out anything more
                    % efficient than the split item (this assumption weakens the bound).
                    fixInUB = LR + KP.Values(i) - KP.Weights(i)*SR;
                    if fixInUB <= GS
                        % Fixing the item to be in cannot result in a
                        % better solution than the greedy solution, so fix
                        % it out.
                        fixes(i) = -1;
                    end
                end
            end
        end
        
        function RK = reducedKnapsack(KP)
            %REDUCEDKNAPSACK Creates new instance without trivially fixable items
            % Creates a new (potentially smaller) knapsack item by removing
            % items that can be "trivially" assigned as in or out of the
            % optimal packing based on the greedy solution and linear
            % relaxation bound.
            fixes = KP.Fixes;
            fixin = find(fixes==1);
            unfix = find(fixes==0);
            RK = Knapsack('temp', 'temp', KP.Weights(unfix), KP.Values(unfix), KP.Capacity - sum(KP.Weights(fixin)));
        end
        
        %----------------------------------------------------------------------
        % Features
        %----------------------------------------------------------------------
        
        function f = DominantPairs(KP)
            %DOMINANTPAIRS Feature (proportion of items where one dominates)
            % Returns the proportion of pairs of items where one dominates
            % the other. Completely unreadable code but it's *fast*.
            % See slower versions below for easier-to-read code which produces
            % the same result, but slower.
            % Lower bound: >= 0
            % Upper bound: < 1
            % Default: 0
            % Source: Martello and Toth (1993)?, Hall and Posner (2007)
            
            v = KP.Values;
            w = KP.Weights;
            kpsize = KP.Size;
            wsw = KP.WSortedWeight;
            vsv = KP.VSortedValue;
            swi = KP.SortedWeightInd;
            svi = KP.SortedValueInd;

            divs = 50;
            indxs = ones(divs,1);
            for i = 2:divs
                indxs(i) = floor((i-1)*KP.Size/divs) + 1;
            end
            if kpsize > divs
                Wmins = [KP.WSortedWeight(indxs);KP.WSortedWeight(end)+1];
                Vmins = [KP.VSortedValue(indxs);KP.VSortedValue(end)+1];
                Wsecs = zeros(kpsize,1);
                %
                Wlistpos = zeros(divs,1);
                Vsecs = zeros(kpsize,1);
                currw = 1;
                currv = 1;
                for i = 1:kpsize
                    if wsw(i) >= Wmins(currw+1)
                        currw = currw + 1;
                    end
                    if vsv(i) >= Vmins(currv+1)
                        currv = currv + 1;
                    end
                    Wsecs(swi(i)) = currw;
                    Vsecs(svi(i)) = currv;
                    Wlistpos(currw) = Wlistpos(currw) + 1;
                end
                
                gridind_map = zeros(kpsize,1);
                countmat = zeros(divs,divs);
                posmat = zeros(divs,divs);
                
                Wlist = (kpsize+1)*ones(divs,max(Wlistpos));
                itemis = 1;
                gridind_pos = 1;
                for d = 1:divs
                    for f = 1:Wlistpos(d)
                        Wlist(d,f) = swi(itemis);
                        itemis = itemis + 1;
                    end
                    [XX,II] = sort(Vsecs(Wlist(d,1:Wlistpos(d))));
                    gridind_map(gridind_pos:(gridind_pos+Wlistpos(d)-1)) = Wlist(d,II)';
                    vindex = 1;
                    currlen = 0;
                    posmat(d,1) = gridind_pos;
                    for ipos = 1:Wlistpos(d)
                        while XX(ipos) > vindex
                            countmat(d,vindex) = currlen;
                            currlen = 0;
                            posmat(d,vindex+1) = gridind_pos;
                            vindex = vindex + 1;
                        end
                        currlen = currlen + 1; 
                        gridind_pos = gridind_pos + 1;
                    end
                    while divs > vindex
                        countmat(d,vindex) = currlen;
                        currlen = 0;
                        posmat(d,vindex+1) = gridind_pos;
                        vindex = vindex + 1;
                    end
                    countmat(d,divs) = currlen;
                    
                end
                
                % Calculate 'obvious' dominant pairs
                cumumat = countmat;
                for i = divs-1:-1:1
                    cumumat(i,:) = cumumat(i,:) + cumumat(i+1,:);
                end
                for i = 2:divs
                    cumumat(:,i) = cumumat(:,i) + cumumat(:,i-1);
                end
                obvious = sum(sum(countmat(1:end-1,2:end) .* cumumat(2:end,1:end-1)));
                % Calculate 'difficult' pairs
                difficult1 = 0;
                difficult1a = 0;
                difficult2 = 0;
                for i = 1:divs
                    for j = divs:-1:1
                        refAC = countmat(i,j);
                        refAP = posmat(i,j);
                        for itm1 = 1:refAC
                            ind1 = gridind_map(refAP + itm1 - 1);
                            for itm2 = (itm1+1):refAC
                                ind2 = gridind_map(refAP + itm2 - 1);
                                if (w(ind1) <= w(ind2) && v(ind1) > v(ind2)) || (w(ind1) < w(ind2) && v(ind1) >= v(ind2))
                                    difficult1 = difficult1 + 1;
                                else
                                    if (w(ind2) <= w(ind1) && v(ind2) > v(ind1)) || (w(ind2) < w(ind1) && v(ind2) >= v(ind1))
                                        difficult1a = difficult1a + 1;
                                    end
                                end
                            end
                            for k = i+1:divs
                                refBC = countmat(k,j);
                                refBP = posmat(k,j);
                                for itm2 = 1:refBC
                                    ind2 = gridind_map(refBP + itm2 - 1);
                                    if (v(ind1) >= v(ind2))
                                        difficult2 = difficult2 + 1;
                                    end
                                end
                            end
                            for k = j-1:-1:1
                                refBC = countmat(i,k);
                                refBP = posmat(i,k);
                                for itm2 = 1:refBC
                                    ind2 = gridind_map(refBP + itm2 - 1);
                                    if (w(ind1) <= w(ind2))
                                        difficult2 = difficult2 + 1;
                                    end
                                end
                            end
                        end
                    end
                end
                f = (obvious+difficult1+difficult1a+difficult2) / (kpsize*(kpsize-1)/2);
            else
                if kpsize > 1 % required for at least one pair to exist
                    count = 0;
                    for i = 1:kpsize
                        for j = i+1:kpsize
                            if (KP.VSortedWeight(i) > KP.VSortedWeight(j) || (KP.WSortedWeight(i) == KP.WSortedWeight(j) && KP.VSortedWeight(i) < KP.VSortedWeight(j)))
                                count = count + 1;
                            end
                        end
                    end
                    f = count / (kpsize*(kpsize-1)/2);
                else
                    f = 0;
                end
            end
        end
        
        function f = DominantPairs_sane(KP)
            %DOMINANTPAIRS_SANE Earlier version of feature code for testing
            % Returns the proportion of pairs of items where one dominates
            % the other. Reasonably fast but not fast enough...
            % Lower bound: >= 0
            % Upper bound: < 1
            % Default: 0
            % Source: Martello and Toth (1993)?, Hall and Posner (2007)
            
            v = KP.Values;
            w = KP.Weights;
            wsw = KP.WSortedWeight;
            vsv = KP.VSortedValue;
            swi = KP.SortedWeightInd;
            svi = KP.SortedValueInd;

            divs = 50;
            indxs = ones(divs,1);
            for i = 2:divs
                indxs(i) = floor((i-1)*KP.Size/divs) + 1;
            end
            if KP.Size > divs
                Wmins = [KP.WSortedWeight(indxs);KP.WSortedWeight(end)+1];
                Vmins = [KP.VSortedValue(indxs);KP.VSortedValue(end)+1];
                Wsecs = zeros(KP.Size,1);
                Vsecs = zeros(KP.Size,1);
                currw = 1;
                currv = 1;
                for i = 1:KP.Size
                    if wsw(i) >= Wmins(currw+1)
                        currw = currw + 1;
                    end
                    if vsv(i) >= Vmins(currv+1)
                        currv = currv + 1;
                    end
                    Wsecs(swi(i)) = currw;
                    Vsecs(svi(i)) = currv;
                end
                gridind = cell(divs);
                for i = 1:KP.Size
                    gridind{Wsecs(i),Vsecs(i)} = [gridind{Wsecs(i),Vsecs(i)};i];
                end
                % Calculate 'obvious' dominant pairs
                countmat = zeros(divs);
                for i = 1:divs
                    for j = 1:divs
                        countmat(i,j) = size(gridind{i,j},1);
                    end
                end
                cumumat = countmat;
                for i = divs-1:-1:1
                    cumumat(i,:) = cumumat(i,:) + cumumat(i+1,:);
                end
                for i = 2:divs
                    cumumat(:,i) = cumumat(:,i) + cumumat(:,i-1);
                end
                obvious = sum(sum(countmat(1:end-1,2:end) .* cumumat(2:end,1:end-1)));
                % Calculate 'difficult' pairs
                difficult1 = 0;
                difficult1a = 0;
                difficult2 = 0;
                for i = 1:divs
                    for j = divs:-1:1
                        tmpgindA = gridind{i,j};
                        for itm1 = 1:countmat(i,j)
                            ind1 = tmpgindA(itm1);
                            for itm2 = (itm1+1):countmat(i,j)
                                ind2 = tmpgindA(itm2);
                                if (w(ind1) <= w(ind2) && v(ind1) > v(ind2)) || (w(ind1) < w(ind2) && v(ind1) >= v(ind2))
                                    difficult1 = difficult1 + 1;
                                else
                                    if (w(ind2) <= w(ind1) && v(ind2) > v(ind1)) || (w(ind2) < w(ind1) && v(ind2) >= v(ind1))
                                        difficult1a = difficult1a + 1;
                                    end
                                end
                            end
                            for k = i+1:divs
                                tmpgindB = gridind{k,j};
                                for itm2 = 1:countmat(k,j)
                                    ind2 = tmpgindB(itm2);
                                    if (v(ind1) >= v(ind2))
                                        difficult2 = difficult2 + 1;
                                    end
                                end
                            end
                            for k = j-1:-1:1
                                tmpgindB = gridind{i,k};
                                for itm2 = 1:countmat(i,k)
                                    ind2 = tmpgindB(itm2);
                                    if (w(ind1) <= w(ind2))
                                        difficult2 = difficult2 + 1;
                                    end
                                end
                            end
                        end
                    end
                end
                f = (obvious+difficult1+difficult1a+difficult2) / (KP.Size*(KP.Size-1)/2);
            else
                if KP.Size > 1 % required for at least one pair to exist
                    count = 0;
                    for i = 1:KP.Size
                        for j = i+1:KP.Size
                            if (KP.VSortedWeight(i) > KP.VSortedWeight(j) || (KP.WSortedWeight(i) == KP.WSortedWeight(j) && KP.VSortedWeight(i) < KP.VSortedWeight(j)))
                                count = count + 1;
                            end
                        end
                    end
                    f = count / (KP.Size*(KP.Size-1)/2);
                else
                    f = 0;
                end
            end
        end
        
        function f = DominantPairs_dumb(KP)
            %DOMINANTPAIRS_DUMB Slow but safe feature code for testing
            % Returns the proportion of pairs of items where one dominates
            % the other. Naive implementation for testing purposes.
            % Lower bound: >= 0
            % Upper bound: < 1
            % Default: 0
            % Source: Martello and Toth (1993)?, Hall and Posner (2007)
            
            if KP.Size > 1 % required for at least one pair to exist
                count = 0;
                counta = 0;
                for i = 1:KP.Size
                    for j = i+1:KP.Size
                        if (KP.Weights(i) <= KP.Weights(j) && KP.Values(i) > KP.Values(j)) || (KP.Weights(i) < KP.Weights(j) && KP.Values(i) >= KP.Values(j))
                            count = count + 1;
                        end
                        if (KP.Weights(j) <= KP.Weights(i) && KP.Values(j) > KP.Values(i)) || (KP.Weights(j) < KP.Weights(i) && KP.Values(j) >= KP.Values(i))
                            counta = counta + 1;
                        end
                    end
                end
                f = (count+counta) / (KP.Size*(KP.Size-1)/2);
            else
                f = 0;
            end
        end
        
        function f = CapacityFraction(KP)
            %CAPACITYFRACTION Feature (kp capacity / sum of weights)
            % Returns the capacity fraction - the ratio between the capacity of
            % the knapsack and the summed total weight of the items,
            % bounded above by 1 (since any further increase to capacity
            % does not affect the problem).
            % Lower bound: >= 0 (knapsack is full)
            % Upper bound: <= 1 (imposed limit)
            % Default: 0
            % Source: Hall and Posner (2007)
            if sum(KP.Weights) > 0
                f = min(KP.Capacity / sum(KP.Weights),1);
            else
                f = 0;
            end
        end
        
        function f = CapacityFractionDifference(KP)
            %CAPACITYFRACTIONDIFFERENCE Feature (|capfrac - 0.5| * 2)
            % Returns double the difference between the capacity fraction and 0.5.
            % Lower bound: >= 0 (CapacityFraction is 0.5)
            % Upper bound: <= 1 (CapacityFraction is 0 or 1)
            % Default: 1 (inherited from CapacityFraction)
            % Source: N/A
            f = abs(KP.CapacityFraction - 0.5) * 2;
        end
        
        function f = CorrelationCoefficient(KP)
            %CORRELATIONCOEFFICIENT Feature (corr. coeff. between weights and values)
            % Returns the correlation coefficient between weights and
            % values.
            % Lower bound: >= -1
            % Upper bound: <= 1
            % Default: 0
            % Source: Hall and Posner (2007)
            if KP.Size > 1
                C = corrcoef(KP.Weights, KP.Values);
                if isfinite(C(1,2))
                    f = C(1,2);
                else
                    f = 0;
                end
            else
                f = 0;
            end
        end
        
        function f = ApproximationGap(KP)
            %APPROXIMATIONGAP Feature (Diff. between trivial bounds)
            % Returns the difference between the value of the greedy
            % feasible solution and linear relaxation upper bound, scaled
            % by the largest profit.
            % Lower bound: >= 0
            % Upper bound: < 1
            % Default: 0
            % Source: Hall and Posner (2007)
            LR = KP.LinearBound;
            GS = KP.GreedyBound;
            SR = KP.SplitRatio;
            if KP.Size > 0
                f = (LR - GS) / max(KP.Values);
            else
                f = 0;
            end
        end
        
        function f = CoeffVarWeights(KP)
            %COEFFVARWEIGHTS Feature (Coefficient of variation of item weights)
            % Returns the coefficient of variation of the weights.
            % Lower bound: >= 0
            % Upper bound: <= sqrt(size)? (In practice, generally on the
            % close order of 1)
            % Default: 0
            % Source: Hall and Posner (2007)
            if isfinite(nanstd(KP.Weights)/nanmean(KP.Weights))
                f = nanstd(KP.Weights)/nanmean(KP.Weights);
                f = atan(f)/(pi/2);
            else
                f = 0;
            end
        end
        
        function f = CoeffVarValues(KP)
            %COEFFVARVALUES Feature (Coefficient of variation of item values)
            % Returns the coefficient of variation of the values.
            % Lower bound: >= 0
            % Upper bound: <= sqrt(size)? (In practice, generally on the
            % close order of 1)
            % Default: 0
            % Source: Hall and Posner (2007)
            if isfinite(nanstd(KP.Values)/nanmean(KP.Values))
                f = nanstd(KP.Values)/nanmean(KP.Values);
                f = atan(f)/(pi/2);
            else
                f = 0;
            end
        end
        
        function f = CoeffVarEffic(KP)
            %COEFFVAREFFIC Feature (Coefficient of variation of item efficiencies)
            % Returns the coefficient of variation of the efficiencies.
            % Lower bound: >= 0
            % Upper bound: <= sqrt(size)? (In practice, generally on the
            % close order of 1)
            % Default: 0
            % Source: Hall and Posner (2007)
            Effs = KP.Values ./ KP.Weights;
            if isfinite(nanstd(Effs)/nanmean(Effs))
                f = nanstd(Effs)/nanmean(Effs);
                f = atan(f/5)/(pi/2);
            else
                f = 0;
            end
        end
        
        function f = SmallerBetterPairs(KP)
            %SMALLERBETTERPAIRS Feature (prop)
            % Returns the proportion of pairs of items where one item is
            % smaller and has better efficiency.
            % This code is slow but the feature wasn't used in GA so it hasn't mattered yet.
            % Could be made faster in the same way as DominantPairs if required.
            % Lower bound: >= 0
            % Upper bound: < 1
            % Default: 0
            % Source: ??? Appears to be unused.
            vsw = KP.VSortedWeight;
            wsw = KP.WSortedWeight;
            
            if KP.Size > 1
                count = 0;
                for i = 1:KP.Size
                    for j = i+1:KP.Size
                        if (vsw(i)/wsw(i) > vsw(j)/wsw(j) || (wsw(i)==wsw(j) && vsw(i)/wsw(i) < vsw(j)/wsw(j)))
                            count = count + 1;
                        end
                    end
                end
                f = count / (KP.Size*(KP.Size-1)/2);
            else
                f = 0;
            end
        end
        
        function f = PossibleItemFixProportion(KP)
            %POSSIBLEITEMFIXPROPORTION Feature (Proportion of items easily fixable)
            % Returns the proportion of items which can be "trivially" assigned
            % as in or out of the optimal packing based on the greedy
            % solution and linear relaxation bound.
            % Lower bound: >= 0
            % Upper bound: < 1
            % Default: 1
            % Source: JTC honours thesis (2013)
            fixes = KP.Fixes;
            
            if KP.Size > 0
                f = nnz(fixes) / KP.Size;
            else
                f = 1;
            end
        end

        function f = SubsetLikeness(KP)
            %SUBSETLIKENESS Feature (Similarity to subset sum instance)
            % Returns the proportion of items which
            % have efficiency (approximately) equal to the split item. All
            % subset sum instances will return 1.
            % Lower bound: => 0
            % Upper bound: <= 1
            % Default: 0
            % Source: Hall and Posner (2004)?, JTC honours thesis (2013)
            LR = KP.LinearBound;
            GS = KP.GreedyBound;
            SR = KP.SplitRatio;
            sum = 0;
            if KP.Size > 1
                for i = 1:KP.Size
                    sum = sum + ((KP.Values(i)/KP.Weights(i) >= 0.9995 * SR) && (KP.Values(i)/KP.Weights(i) <= 1.0005 * SR));
                end
                if isfinite(sum)
                    f = (sum) / (KP.Size);
                else
                    f = 0;
                end
            else
                f = 0;
            end
        end
        
        function f = BZMeasure(KP)
            %BZMEASURE Feature (Modified version of Balas-Zemel Measure)
            % Computes the modified Balas and Zemel measure.
            % Lower bound: => 0
            % Upper bound: <= 1e10 (artifical limit), postprocessed based on max over all instances
            % Default: 0
            % Source: Balas and Zemel (1977), Hall and Posner (2004)
            % WARNING: Dependent on knapsack size.
            LR = KP.LinearBound;
            GS = KP.GreedyBound;
            SR = KP.SplitRatio;
            
            if (LR - GS) < 1
                f = 0;
            else
                bz = 2*(LR - GS) / max(abs(KP.Values-KP.Weights*SR));
                f = min(bz, 1e10);
				f = atan(f/500)/(pi/2);
            end
        end
        
        function f = MaxCardinality(KP)
            %MAXCARDINALITY Feature (Maximum possible no of items in opt. soln.)
            % Returns the maximum proportion of items that can be feasibly
            % placed in the knapsack, based on a simple heuristic.
            % Lower bound: >= 0
            % Upper bound: <= 1
            % Default: 1
            % Source: Martello and Toth (1993)
            i = 1;
            Ksum = 0;
            C = KP.Capacity;
            SW = KP.WSortedWeight;
            while i <= KP.Size && Ksum < C
                Ksum = Ksum + SW(i,1);
                if Ksum <= C
                    i = i+1;
                end
            end
            f = min(i/(KP.Size),1);
        end
        
        function f = MinCardinality(KP)
            %MINCARDINALITY Feature (Minimum possible no of items in opt. soln.)
            % Returns the minimum proportion of items that can be
            % placed in the knapsack such that no further items can be
            % added, based on a simple heuristic.
            % Lower bound: >= 0
            % Upper bound: <= 1
            % Default: 1
            % Source: Martello and Toth (1993)
            i = 0;
            Ksum = 0;
            C = KP.Capacity;
            SW = KP.WSortedWeight;
            while i < KP.Size && Ksum < C
                Ksum = Ksum + SW(KP.Size-i,1);
                if Ksum <= C
                    i = i+1;
                end
            end
            f = min(i/(KP.Size),1);
        end
        
        function f = FirstWeight(KP)
            %FIRSTWEIGHT Feature (Lightest weight)
            % Returns the weight of the lightest item divided by the
            % heaviest weight.
            % Lower bound: >= 0
            % Upper bound: <= 1 (typically much smaller)
            % Default: 1
            % Source: Chung (1998)
            if KP.Size > 0
                SW = KP.WSortedWeight;
                f = SW(1,1) / max(KP.Weights);
            else
                f = 0;
            end
        end
        
        function f = FirstValue(KP)
            %FIRSTVALUE Feature (Least profitable profit)
            % Returns the weight of the least valuable item divided by the
            % largest value.
            % Lower bound: >= 0
            % Upper bound: <= 1 (typically much smaller)
            % Default: 1
            % Source: ???
            if KP.Size > 0
                f = min(KP.Values) / max(KP.Values);
            else
                f = 0;
            end
        end
        
        function f = CardinalityGap(KP)
            %CARDINALITYGAP Feature (Difference between cardinality bounds)
            % Lower bound: >= 0
            % Upper bound: <= 1 (probably less)
            % Default: ?
            % Source: Sam's research
            A = KP.MaxCardinality;
            B = KP.MinCardinality;
            
            f = (A-B);
        end
        
        function f = PolyfitLinear(KP)
            %POLYFITLINEAR Feature (Quality of fit to linear trend)
            % Returns the quality of fit of weights and values to a linear trend.
            % Lower bound: >= 0
            % Upper bound: <= 1
            % Default: Limit to closer bound, 0 if NaN
            % Note that -Inf can appear for degenerate problems!
            % Source: Sam's research
            W = KP.Weights;
            V = KP.Values;
            
            p = polyfit(W,V,1);
            Vp = polyval(p,W);
            
            ft = 1 - sum((V-Vp).^2)/sum((V-mean(V)).^2);
            
            f = max(min(ft,1),0);
        end
        
        function f = PolyfitQuadratic(KP)
            %POLYFITQUADRATIC Feature (Quality of fit to quadratic trend)
            % Returns the quality of fit of weights and values to a quadratic trend.
            % Lower bound: >= 0
            % Upper bound: <= 1
            % Default: Limit to closer bound, 0 if NaN
            % Source: Sam's research
            W = KP.Weights;
            V = KP.Values;
            
            p = polyfit(W,V,2);
            Vp = polyval(p,W);
            
            ft = 1 - sum((V-Vp).^2)/sum((V-mean(V)).^2);
            
            f = max(min(ft,1),0);
        end
        
        function f = PolyfitCubic(KP)
             %POLYFITCUBIC Feature (Quality of fit to cubic trend)
            % Returns the quality of fit of weights and values to a cubic trend.
            % Lower bound: >= 0
            % Upper bound: <= 1
            % Default: Limit to closer bound, 0 if NaN
            % Source: Sam's research
            W = KP.Weights;
            V = KP.Values;
            
            p = polyfit(W,V,3);
            Vp = polyval(p,W);
            
            ft = 1 - sum((V-Vp).^2)/sum((V-mean(V)).^2);
            f = max(min(ft,1),0);
        end
        
        function f = BeforeAfterRatio(KP)   
            %BEFOREAFTERRATIO Feature (Proportion of items before/after split item)
            % Returns the proportion of items better than split to the
            % others.
            % Lower bound: >= 0
            % Upper bound: < KP.Size (if the split item is the worst)
            % Default: ???
            % Source: Sam's research
            
            LR = KP.LinearBound;
            GS = KP.GreedyBound;
            SR = KP.SplitRatio;

            E = KP.ESortedEffic;
            
            k = 0;
            l = 0;
            if KP.Size > 0
				for i = 1:length(E)
					
					if E(i) > SR
						
						k = k+1; %items better (before) split
						
					else
						
						l = l+1; %items equal to or better than split
					end
				end
				
				if k > l
                    f = atan(k/l - 1)/pi + 0.5;
                else
                    f = atan(-(l/k - 1))/pi + 0.5;
                end
			else
				f = 0;
			end
            
        end
        
        function f = EvenOddLikeness(KP)
            %SPLITITEMGCD Feature (Similarity to even-odd or spanner instance)
            % Multiply SubsetLikeness by 10 if GCD of weights of split items is >1. 
            
            LR = KP.LinearBound;
            GS = KP.GreedyBound;
            SR = KP.SplitRatio;
            sum = 0;
            g = -1;
            if KP.Size > 1
                for i = 1:KP.Size
                    if ((KP.Values(i)/KP.Weights(i) >= 0.9995 * SR) && (KP.Values(i)/KP.Weights(i) <= 1.0005 * SR))
                        sum = sum + 1;
                        if g > 0
                            g = gcd(g,KP.Weights(i));
                        else
                            g = KP.Weights(i);
                        end
                    end
                end
                if isfinite(sum)
                    f = (sum) / (KP.Size);
                else
                    f = 0;
                end
                if g > 1
                    f = f * 10;
                end
            else
                f = 0;
            end
            
%             LR = KP.LinearBound;
%             GS = KP.GreedyBound;
%             SR = KP.SplitRatio;
%             sum = 0;
%             closesum = 0;
%             g = -1;
%             if KP.Size > 1
%                 for i = 1:KP.Size
%                     if ((KP.Values(i)/KP.Weights(i) >= 0.9995 * SR) && (KP.Values(i)/KP.Weights(i) <= 1.0005 * SR))
%                         sum = sum + 1;
%                         if g > 0
%                             g = gcd(g,KP.Weights(i));
%                         else
%                             g = KP.Weights(i);
%                         end
%                     else
%                         if ((KP.Values(i)/KP.Weights(i) >= 0.90 * SR) && (KP.Values(i)/KP.Weights(i) <= 1.10 * SR))
%                             closesum = closesum + 0.5;
%                         end
%                         if ((KP.Values(i)/KP.Weights(i) >= 0.95 * SR) && (KP.Values(i)/KP.Weights(i) <= 1.05 * SR))
%                             closesum = closesum + 0.5;
%                         end
%                     end
%                     %sum = sum + ((KP.Values(i)/KP.Weights(i) >= 0.9995 * SR) && (KP.Values(i)/KP.Weights(i) <= 1.0005 * SR));
%                 end
%                 if isfinite(sum)
%                     f = (sum * closesum) / (KP.Size * KP.Size);
%                 else
%                     f = 0;
%                 end
%                 if g > 1
%                     f = f * 10;
%                 end
%             else
%                 f = 0;
%             end
        end
        
        function f = GreedyUnusedCapacity(KP)
            %GREEDYUNUSEDCAPACITY Feature (Capacity unused by greedy solution)
            GW = KP.GreedyWaste;
            f = 0;
            if KP.Size > 1
                wdiff = KP.WSortedWeight(2:end) - KP.WSortedWeight(1:end-1);
                if max(wdiff) > 0
                    minjump = min(wdiff(wdiff ~= 0));
                    f = GW/minjump;
                    f = atan(f/100)/(pi/2);
                end
            end
        end
        
        %------------------------------------------------------------------
        % Debugging tools
        %----------------------------------------------------------------------
        
        function plotWV(KP)
            %PLOTWV Plots weight vs value of items
            figure
            scatter(KP.Weights, KP.Values ,'filled')
        end
        
        function plotWE(KP)
            %PLOTWE Plots weight vs efficiencies of items
            figure
            hold on
            scatter(KP.Weights, KP.Values./KP.Weights ,'filled')
            plot([0,1000],[KP.SplitRatio, KP.SplitRatio])
            hold off
        end
		
		%------------------------------------------------------------------
        % Retrieve feature vector
        %----------------------------------------------------------------------
		
		function [vector] = getFeatureVector(KP,indices)
		%GETFEATUREVECTOR Summary of this function goes here
		%   Detailed explanation goes here

			RK = KP.reducedKnapsack();
			
			flist = Knapsack.featureList;
			
			rflist = Knapsack.reducedFeatureList;
			
			vector = zeros(1,length(indices));
			
			for i = 1:length(indices)
				if indices(i) <= size(flist,1)
					vector(i) = flist{indices(i),2}(KP);
				else
					vector(i) = rflist{indices(i)-size(flist,1),2}(RK);
				end
			end

		end
    end
    
    methods (Static)
        function KP = readFromFile(path, filename)
            %READFROMFILE Read knapsack instance from file 
            % Reads file previously created by writeToFile. Used by evaluateFeatures.m
            strcat(filename, path)
            f = fopen(strcat(filename, path), 'r');
            name = fscanf(f,'%s',1);
            class = fscanf(f,'%s',1);
            capacity = fscanf(f,'%d', 1);
            tmp = fscanf(f,'%f %f', [2 inf])'; %FIX ME
            weights = max(tmp(:,1),0);
            values = max(ceil(tmp(:,2)),0);
            KP = Knapsack(name, class, weights, values, capacity);
            fclose(f);
        end
        
        function list = featureList()
            %FEATURELIST Return a list of feature functions, their names, and their theoretical bounds.
            list = { 'DominantPairs', @DominantPairs, 0, 1;
                'SmallerBetterPairs', @SmallerBetterPairs, 0, 1;
                'CapacityFraction', @CapacityFraction, 0, 1;
                'CapFracDistanceFromOneHalf', @CapacityFractionDifference, 0, 1;
                'CorrelationCoefficient', @CorrelationCoefficient, -1, 1;
                'ApproximationGap', @ApproximationGap, 0, 1;
                'CoeffVarWeights', @CoeffVarWeights, 0, 1; %scaled
                'CoeffVarProfits', @CoeffVarValues, 0, 1; %scaled
                'CoeffVarEffics', @CoeffVarEffic, 0, 1; %scaled
                'PossibleItemFixProportion', @PossibleItemFixProportion, 0, 1;
                'SubsetSumLikeness', @SubsetLikeness, 0, 1;
                'ModBalasZemelMeasure', @BZMeasure, 0, 1; %scaled
                'MaxCardinality', @MaxCardinality, 0, 1;
                'MinCardinality', @MinCardinality 0, 1;
                'CardinalityGap', @CardinalityGap 0, 1;
                'FirstWeight', @FirstWeight, 0, 1; %variable, sort of
                'FirstProfit', @FirstValue, 0, 1;
                'PolyfitLinear', @PolyfitLinear, 0, 1;
                'PolyfitQuadratic', @PolyfitQuadratic, 0, 1;
                'PolyfitCubic', @PolyfitCubic, 0, 1;
                'BeforeAfterSplitRatio', @BeforeAfterRatio, 0, 1; %variable
                'EvenOddLikeness', @EvenOddLikeness, 0, 10;
                'GreedyUnusedCapacity', @GreedyUnusedCapacity, 0, 1;
                };
        end
        
        function list = reducedFeatureList()
            %REDUCEDFEATURELIST Return a list of features appropriate for reduced knapsac
            list = { 'DominantPairs', @DominantPairs, 0, 1;
                'SmallerBetterPairs', @SmallerBetterPairs, 0, 1;
                'CapacityFraction', @CapacityFraction, 0, 1;
                'CapFracDistanceFromOneHalf', @CapacityFractionDifference, 0, 1;
                'CorrelationCoefficient', @CorrelationCoefficient, -1, 1;
                'ApproximationGap', @ApproximationGap, 0, 1;
                'CoeffVarWeights', @CoeffVarWeights, 0, 1; %scaled
                'CoeffVarProfits', @CoeffVarValues, 0, 1; %scaled
                'CoeffVarEffics', @CoeffVarEffic, 0, 1; %scaled
                %'PossibleItemFixProportion', @PossibleItemFixProportion, 0, 1;
                'SubsetLikeness', @SubsetLikeness, 0, 1;
                'ModBalasZemelMeasure', @BZMeasure, 0, 1; %scaled
                'MaxCardinality', @MaxCardinality, 0, 1;
                'MinCardinality', @MinCardinality 0, 1;
                'CardinalityGap', @CardinalityGap 0, 1;
                'FirstWeight', @FirstWeight, 0, 1; %variable, sort of
                'FirstProfit', @FirstValue, 0, 1;
                'PolyfitLinear', @PolyfitLinear, 0, 1;
                'PolyfitQuadratic', @PolyfitQuadratic, 0, 1;
                'PolyfitCubic', @PolyfitCubic, 0, 1;
                'BeforeAfterSplitRatio', @BeforeAfterRatio, 0, 1; %variable
                'GreedyUnusedCapacity', @GreedyUnusedCapacity, 0, 1;
                };
        end
    end
    
end

